import {
  Body,
  Controller,
  HttpCode,
  NotImplementedException,
  Post,
  Req,
  UnauthorizedException,
  UseGuards,
} from '@nestjs/common';
import { LoginPayload } from './models/payloads/login.payload';
import { LocalLoginDto } from './models/dto/local-login.dto';
import { UserEntity } from './models/entities/user.entity';
import { JwtScope } from './models/enums/jwt-scope';
import { LoginService } from './login.service';
import { ApiBearerAuth, ApiTags } from '@nestjs/swagger';
import { AuthGuard } from '@nestjs/passport';
import { UsersService } from '../../../modules/users/users.service';

@Controller('auth/login')
@ApiTags('Auth')
export class LoginController {
  constructor(
    private readonly loginService: LoginService,
    private readonly usersService: UsersService,
  ) {}

  /**
   * Logs in the user using email and password
   *
   * @throws {401} Invalid password
   * @throws {404} User not found
   * @throws {500} Internal server error
   */
  @Post('')
  async login(@Body() body: LocalLoginDto): Promise<LoginPayload> {
    const user: UserEntity = await this.loginService.validateUser(
      body.email,
      body.password,
    );
    if (!(await this.loginService.isUserVerified(user.id)))
      throw new UnauthorizedException('User not verified');
    const token: string = this.loginService.generateToken(
      user.id,
      user.tokenId,
      JwtScope.USAGE,
    );
    return new LoginPayload({
      user,
      token,
    });
  }

  /**
   * Handles the login callback for the authentication process (Magic Link, Passkey, 2FA)
   *
   * @throws {401} Unauthorized
   * @throws {500} Internal server error
   */
  @Post('callback')
  @UseGuards(AuthGuard('auth-jwt'))
  @ApiBearerAuth()
  loginCallback(@Req() req: any): Promise<LoginPayload> {
    // TODO: Fix this line
    req = req.user;
    if (req.scope === JwtScope.MAGIC) {
      // Check for 2FA/Passkey
      const token: string = this.loginService.generateToken(
        req.user.id,
        req.user.tokenId,
        JwtScope.USAGE,
      );
      return new LoginPayload({
        user: req.user,
        token,
      });
    }
    const authToken: string = this.loginService.generateToken(
      req.user.id,
      req.user.tokenId,
      JwtScope.USAGE,
    );
    return new LoginPayload({
      user: req.user,
      token: authToken,
    });
  }

  @Post('passkey')
  loginPasskey() {
    // Generate JWT token
    throw new NotImplementedException();
  }
}
